<?php

namespace Drupal\devportal_api_entities\Entity;

use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\devportal_api_entities\Traits\APIDocRefTrait;
use Drupal\devportal_api_reference\Traits\APIRefTrait;
use Drupal\devportal_api_entities\Traits\APIExtDocRefTrait;
use Drupal\devportal_api_entities\Traits\AutoLabelTrait;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\devportal_api_entities\APIGlobalSchemaInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\devportal_api_entities\Traits\APIVersionTagRefTrait;
use Drupal\devportal\Traits\URLRouteParametersTrait;;

/**
 * Defines the API Global Schema entity class.
 *
 * @ContentEntityType(
 *   id = "api_global_schema",
 *   label = @Translation("API Global Schema"),
 *   handlers = {
 *     "storage" = "Drupal\devportal_api_entities\APIGlobalSchemaStorage",
 *     "list_builder" = "Drupal\devportal_api_entities\APIGlobalSchemaListBuilder",
 *     "view_builder" = "Drupal\devportal\DevportalContentEntityViewBuilder",
 *     "views_data" = "Drupal\devportal_api_entities\APIGlobalSchemaViewsData",
 *     "form" = {
 *       "default" = "Drupal\devportal\Form\DevportalContentEntityForm",
 *       "add" = "Drupal\devportal\Form\DevportalContentEntityForm",
 *       "edit" = "Drupal\devportal\Form\DevportalContentEntityForm",
 *       "delete" = "Drupal\devportal\Form\DevportalContentEntityDeleteForm",
 *     },
 *     "inline_form" = "Drupal\devportal_api_entities\Form\DevportalInlineForm",
 *     "route_provider" = {
 *       "html" = "Drupal\devportal_api_entities\APIGlobalSchemaHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\devportal_api_entities\APIGlobalSchemaAccessControlHandler",
 *     "translation" = "Drupal\devportal_api_entities\APIGlobalSchemaTranslationHandler",
 *   },
 *   admin_permission = "administer api global schemas",
 *   fieldable = TRUE,
 *   base_table = "api_global_schema",
 *   data_table = "api_global_schema_field_data",
 *   field_ui_base_route = "entity.api_global_schema_type.edit_form",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "type",
 *     "uuid" = "uuid",
 *     "revision" = "vid",
 *     "langcode" = "langcode",
 *     "label" = "auto_label",
 *   },
 *   api_extra_info = {
 *     "auto_label" = {
 *       "auto_label" = "auto_label",
 *       "autogenerated_label" = "autogenerated_label",
 *     },
 *     "auto_label_source" = "name",
 *     "api_doc" = "api_doc",
 *     "api_ref" = "api_ref",
 *     "api_ext_doc" = "api_ext_doc",
 *     "api_version_tag" = "api_version_tag",
 *   },
 *   bundle_entity_type = "api_global_schema_type",
 *   bundle_label = @Translation("API Global Schema type"),
 *   revision_table = "api_global_schema_revision",
 *   revision_data_table = "api_global_schema_field_revision",
 *   show_revision_ui = TRUE,
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_user",
 *     "revision_created" = "revision_created",
 *     "revision_log_message" = "revision_log",
 *   },
 *   links = {
 *     "canonical" = "/api_global_schema/{api_global_schema}",
 *     "add-page" = "/api_global_schema/add",
 *     "add-form" = "/api_global_schema/add/{api_global_schema_type}",
 *     "edit-form" = "/api_global_schema/{api_global_schema}/edit",
 *     "delete-form" = "/api_global_schema/{api_global_schema}/delete",
 *     "collection" = "/admin/content/api_global_schema",
 *     "version-history" = "/api_global_schema/{api_global_schema}/revisions",
 *     "revision" = "/api_global_schema/{api_global_schema}/revisions/{api_global_schema_revision}/view",
 *     "revision_revert" = "/api_global_schema/{api_global_schema}/revisions/{api_global_schema_revision}/revert",
 *     "revision_delete" = "/api_global_schema/{api_global_schema}/revisions/{api_global_schema_revision}/delete",
 *     "multiple_delete_confirm" = "/admin/content/api_global_schema/delete",
 *     "translation_revert" = "/api_global_schema/{api_global_schema}/revisions/{api_global_schema_revision}/revert/{langcode}",
 *   },
 *   translatable = TRUE,
 * )
 */
class APIGlobalSchema extends RevisionableContentEntityBase implements APIGlobalSchemaInterface {

  use EntityChangedTrait;
  use AutoLabelTrait;
  use APIDocRefTrait;
  use APIRefTrait;
  use APIExtDocRefTrait;
  use APIVersionTagRefTrait;
  use URLRouteParametersTrait;

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getValue() {
    return $this->get('value')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($value) {
    $this->set('value', $value);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    // Generate auto label.
    $this->autoLabelPreSave();
  }

  /**
   * {@inheritdoc}
   */
  public function preSaveRevision(EntityStorageInterface $storage, \stdClass $record) {
    parent::preSaveRevision($storage, $record);

    if (!$this->isNewRevision() && isset($this->original) && (!isset($record->revision_log) || $record->revision_log === '')) {
      // If we are updating an existing APIGlobalSchema without adding a
      // new revision, we need to make sure $entity->revision_log is reset
      // whenever it is empty. Therefore, this code allows us to avoid
      // clobbering an existing log entry with an empty one.
      $record->revision_log = $this->original->revision_log->value;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    /** @var \Drupal\Core\Field\BaseFieldDefinition[] $fields */
    $fields = parent::baseFieldDefinitions($entity_type);

    // Add the auto label field.
    $fields += static::autoLabelBaseFieldDefinitions($entity_type);

    // Add the API Documentation field.
    $fields += static::apiDocBaseFieldDefinitions($entity_type);
    $fields['api_doc']->setDescription(t('API Documentation referenced from API Global Schema.'));

    // Add the API Reference field.
    $fields += static::apiRefBaseFieldDefinitions($entity_type);
    $fields['api_ref']->setDescription(t('API Reference referenced from API Global Schema.'));

    // Add the API External documentation field.
    $fields += static::apiExtDocBaseFieldDefinitions($entity_type);
    $fields['api_ext_doc']->setDescription(t('API External Documentation referenced from API Global Schema.'));

    // Add the API Version Tag field.
    $fields += static::apiVersionTagBaseFieldDefinitions($entity_type);
    $fields['api_version_tag']->setDescription(t('API Version Tag referenced from API Global Schema.'));

    $fields['id']->setDescription(t('The API Global Schema ID.'));

    $fields['uuid']->setDescription(t('The API Global Schema UUID.'));

    $fields['vid']->setDescription(t('The API Global Schema revision ID.'));

    $fields['langcode']->setDescription(t('The API Global Schema language code.'));

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setRequired(TRUE)
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE);

    // @TODO Serialized JSON blob.
    $fields['value'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Value'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE)
      ->setDescription(t('Value of the API Global Schema.'))
      // @FIXME Does it need to be displayed, eg. via a special field formatter?
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'text_default',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE)
      // @FIXME: Does it need any form widget at all? If not, set to read-only.
      ->setDisplayOptions('form', [
        'type' => 'text_textfield',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the API Global Schema was last edited.'))
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    $label = parent::label();
    if (empty($label)) {
      $label = $this->generateAutoLabel();
    }
    return $label;
  }

}
