<?php

namespace Drupal\devportal_api_entities\Entity;

use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\devportal_api_reference\Traits\APIRefTrait;
use Drupal\devportal_api_entities\Traits\AutoLabelTrait;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\devportal_api_entities\APIMetaParamInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\devportal_api_entities\Traits\APIVersionTagRefTrait;
use Drupal\devportal\Traits\URLRouteParametersTrait;;
use Drupal\devportal_api_entities\Traits\VendorExtensionTrait;

/**
 * Defines the API Meta Parameter entity class.
 *
 * @ContentEntityType(
 *   id = "api_meta_param",
 *   label = @Translation("API Meta Parameter"),
 *   handlers = {
 *     "storage" = "Drupal\devportal_api_entities\APIMetaParamStorage",
 *     "list_builder" = "Drupal\devportal_api_entities\APIMetaParamListBuilder",
 *     "view_builder" = "Drupal\devportal\DevportalContentEntityViewBuilder",
 *     "views_data" = "Drupal\devportal_api_entities\APIMetaParamViewsData",
 *     "form" = {
 *       "default" = "Drupal\devportal_api_entities\APIMetaParamForm",
 *       "add" = "Drupal\devportal_api_entities\APIMetaParamForm",
 *       "edit" = "Drupal\devportal_api_entities\APIMetaParamForm",
 *       "delete" = "Drupal\devportal\Form\DevportalContentEntityDeleteForm",
 *     },
 *     "inline_form" = "Drupal\devportal_api_entities\Form\DevportalInlineForm",
 *     "route_provider" = {
 *       "html" = "Drupal\devportal_api_entities\APIMetaParamHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\devportal_api_entities\APIMetaParamAccessControlHandler",
 *     "translation" = "Drupal\devportal_api_entities\APIMetaParamTranslationHandler",
 *   },
 *   admin_permission = "administer api meta params",
 *   fieldable = TRUE,
 *   base_table = "api_meta_param",
 *   data_table = "api_meta_param_field_data",
 *   field_ui_base_route = "entity.api_meta_param_type.edit_form",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "type",
 *     "uuid" = "uuid",
 *     "revision" = "vid",
 *     "langcode" = "langcode",
 *     "label" = "auto_label",
 *   },
 *   api_extra_info = {
 *     "auto_label" = {
 *       "auto_label" = "auto_label",
 *       "autogenerated_label" = "autogenerated_label",
 *     },
 *     "vendor_extension" = "extensions",
 *     "auto_label_source" = "name",
 *     "api_ref" = "api_ref",
 *     "api_version_tag" = "api_version_tag",
 *   },
 *   bundle_entity_type = "api_meta_param_type",
 *   bundle_label = @Translation("API Meta Parameter type"),
 *   revision_table = "api_meta_param_revision",
 *   revision_data_table = "api_meta_param_field_revision",
 *   show_revision_ui = TRUE,
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_user",
 *     "revision_created" = "revision_created",
 *     "revision_log_message" = "revision_log",
 *   },
 *   links = {
 *     "canonical" = "/api_meta_param/{api_meta_param}",
 *     "add-page" = "/api_meta_param/add",
 *     "add-form" = "/api_meta_param/add/{api_meta_param_type}",
 *     "edit-form" = "/api_meta_param/{api_meta_param}/edit",
 *     "delete-form" = "/api_meta_param/{api_meta_param}/delete",
 *     "collection" = "/admin/content/api_meta_param",
 *     "version-history" = "/api_meta_param/{api_meta_param}/revisions",
 *     "revision" = "/api_meta_param/{api_meta_param}/revisions/{api_meta_param_revision}/view",
 *     "revision_revert" = "/api_meta_param/{api_meta_param}/revisions/{api_meta_param_revision}/revert",
 *     "revision_delete" = "/api_meta_param/{api_meta_param}/revisions/{api_meta_param_revision}/delete",
 *     "multiple_delete_confirm" = "/admin/content/api_meta_param/delete",
 *     "translation_revert" = "/api_meta_param/{api_meta_param}/revisions/{api_meta_param_revision}/revert/{langcode}",
 *   },
 *   translatable = TRUE,
 *   constraints = {
 *     "APIMetaParam" = {},
 *   },
 * )
 */
class APIMetaParam extends RevisionableContentEntityBase implements APIMetaParamInterface {

  use EntityChangedTrait;
  use AutoLabelTrait;
  use APIRefTrait;
  use APIVersionTagRefTrait;
  use URLRouteParametersTrait;
  use VendorExtensionTrait;

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    // Generate auto label.
    $this->autoLabelPreSave();
  }

  /**
   * {@inheritdoc}
   */
  public function preSaveRevision(EntityStorageInterface $storage, \stdClass $record) {
    parent::preSaveRevision($storage, $record);

    if (!$this->isNewRevision() && isset($this->original) && (!isset($record->revision_log) || $record->revision_log === '')) {
      // If we are updating an existing APIMetaParam without adding a
      // new revision, we need to make sure $entity->revision_log is reset
      // whenever it is empty. Therefore, this code allows us to avoid
      // clobbering an existing log entry with an empty one.
      $record->revision_log = $this->original->revision_log->value;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    /** @var \Drupal\Core\Field\BaseFieldDefinition[] $fields */
    $fields = parent::baseFieldDefinitions($entity_type);

    // Add the auto label field.
    $fields += static::autoLabelBaseFieldDefinitions($entity_type);

    // Add the vendor extension fields.
    $fields += static::vendorExtensionBaseFieldDefinitions($entity_type);

    // Add the API Reference field.
    $fields += static::apiRefBaseFieldDefinitions($entity_type);
    $fields['api_ref']->setDescription(t('API Reference referenced from API Meta Parameter.'));

    // Add the API Version Tag field.
    $fields += static::apiVersionTagBaseFieldDefinitions($entity_type);
    $fields['api_version_tag']->setDescription(t('API Version Tag referenced from API Meta Parameter.'));

    $fields['id']->setDescription(t('The API Meta Parameter ID.'));

    $fields['uuid']->setDescription(t('The API Meta Parameter UUID.'));

    $fields['vid']->setDescription(t('The API Meta Parameter revision ID.'));

    $fields['langcode']->setDescription(t('The API Meta Parameter language code.'));

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setRequired(TRUE)
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['param_in'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('In'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE)
      ->setSettings([
        'allowed_values' => [
          'path' => 'Path',
          'body' => 'Body',
          'query' => 'Query',
          'header' => 'Header',
          'formData' => 'Form data',
        ]
      ])
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['api_path_param'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Path parameter'))
      ->setDescription(t('API Path Parameter referenced from API Meta Parameter.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'target_type' => 'api_path_param',
        'handler' => 'default',
        'handler_settings' => [
          'sort' => [
            'field' => '_none',
          ],
          'auto_create' => FALSE,
          'auto_create_bundle' => '',
        ],
      ])
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_label',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 5,
        'settings' => [
          'form_mode' => 'default',
          'override_labels' => TRUE,
          // @FIXME Should these use $this->>t()?
          'label_singular' => 'API Path Parameter',
          'label_plural' => 'API Path Parameters',
          'allow_new' => TRUE,
          'allow_existing' => FALSE,
          'match_operator' => 'CONTAINS',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['api_body_param'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Body parameter'))
      ->setDescription(t('API Body Parameter referenced from API Meta Parameter.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'target_type' => 'api_body_param',
        'handler' => 'default',
        'handler_settings' => [
          'sort' => [
            'field' => '_none',
          ],
          'auto_create' => FALSE,
          'auto_create_bundle' => '',
        ],
      ])
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_label',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 5,
        'settings' => [
          'form_mode' => 'default',
          'override_labels' => TRUE,
          // @FIXME Should these use $this->>t()?
          'label_singular' => 'API Body Parameter',
          'label_plural' => 'API Body Parameters',
          'allow_new' => TRUE,
          'allow_existing' => FALSE,
          'match_operator' => 'CONTAINS',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['api_query_param'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Query parameter'))
      ->setDescription(t('API Query Parameter referenced from API Meta Parameter.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'target_type' => 'api_query_param',
        'handler' => 'default',
        'handler_settings' => [
          'sort' => [
            'field' => '_none',
          ],
          'auto_create' => FALSE,
          'auto_create_bundle' => '',
        ],
      ])
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_label',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 5,
        'settings' => [
          'form_mode' => 'default',
          'override_labels' => TRUE,
          // @FIXME Should these use $this->>t()?
          'label_singular' => 'API Query Parameter',
          'label_plural' => 'API Query Parameters',
          'allow_new' => TRUE,
          'allow_existing' => FALSE,
          'match_operator' => 'CONTAINS',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['api_header_param'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Header parameter'))
      ->setDescription(t('API Header Parameter referenced from API Meta Parameter.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'target_type' => 'api_header_param',
        'handler' => 'default',
        'handler_settings' => [
          'sort' => [
            'field' => '_none',
          ],
          'auto_create' => FALSE,
          'auto_create_bundle' => '',
        ],
      ])
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_label',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 5,
        'settings' => [
          'form_mode' => 'default',
          'override_labels' => TRUE,
          // @FIXME Should these use $this->>t()?
          'label_singular' => 'API Header Parameter',
          'label_plural' => 'API Header Parameters',
          'allow_new' => TRUE,
          'allow_existing' => FALSE,
          'match_operator' => 'CONTAINS',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['api_form_data_param'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Form data parameter'))
      ->setDescription(t('API Form Data Parameter referenced from API Meta Parameter.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'target_type' => 'api_form_data_param',
        'handler' => 'default',
        'handler_settings' => [
          'sort' => [
            'field' => '_none',
          ],
          'auto_create' => FALSE,
          'auto_create_bundle' => '',
        ],
      ])
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_label',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 5,
        'settings' => [
          'form_mode' => 'default',
          'override_labels' => TRUE,
          // @FIXME Should these use $this->>t()?
          'label_singular' => 'API Form Data Parameter',
          'label_plural' => 'API Form Data Parameters',
          'allow_new' => TRUE,
          'allow_existing' => FALSE,
          'match_operator' => 'CONTAINS',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the API Meta Parameter was last edited.'))
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    $label = parent::label();
    if (empty($label)) {
      $label = $this->generateAutoLabel();
    }
    return $label;
  }

}
