<?php

namespace Drupal\devportal_api_entities\Entity;

use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\devportal_api_entities\Traits\AllowEmptyValueTrait;
use Drupal\devportal_api_reference\Traits\APIRefTrait;
use Drupal\devportal_api_entities\Traits\AutoLabelTrait;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\devportal_api_entities\APIQueryParamInterface;
use Drupal\devportal_api_entities\Traits\MethodParameterTrait;
use Drupal\devportal_api_entities\Traits\ItemTrait;
use Drupal\devportal_api_entities\Traits\APIVersionTagRefTrait;
use Drupal\devportal\Traits\URLRouteParametersTrait;;

/**
 * Defines the API Query Param entity class.
 *
 * @ContentEntityType(
 *   id = "api_query_param",
 *   label = @Translation("API HTTP Method Query Parameter"),
 *   handlers = {
 *     "storage" = "Drupal\devportal_api_entities\APIQueryParamStorage",
 *     "list_builder" = "Drupal\devportal_api_entities\APIQueryParamListBuilder",
 *     "view_builder" = "Drupal\devportal\DevportalContentEntityViewBuilder",
 *     "views_data" = "Drupal\devportal_api_entities\APIQueryParamViewsData",
 *     "form" = {
 *       "default" = "Drupal\devportal\Form\DevportalContentEntityForm",
 *       "add" = "Drupal\devportal\Form\DevportalContentEntityForm",
 *       "edit" = "Drupal\devportal\Form\DevportalContentEntityForm",
 *       "delete" = "Drupal\devportal\Form\DevportalContentEntityDeleteForm",
 *     },
 *     "inline_form" = "Drupal\devportal_api_entities\Form\DevportalInlineForm",
 *     "route_provider" = {
 *       "html" = "Drupal\devportal_api_entities\APIQueryParamHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\devportal_api_entities\APIQueryParamAccessControlHandler",
 *     "translation" = "Drupal\devportal_api_entities\APIQueryParamTranslationHandler",
 *   },
 *   admin_permission = "administer api query params",
 *   fieldable = TRUE,
 *   base_table = "api_query_param",
 *   data_table = "api_query_param_field_data",
 *   field_ui_base_route = "entity.api_query_param_type.edit_form",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "type",
 *     "uuid" = "uuid",
 *     "revision" = "vid",
 *     "langcode" = "langcode",
 *     "label" = "auto_label",
 *   },
 *   api_extra_info = {
 *     "auto_label" = {
 *       "auto_label" = "auto_label",
 *       "autogenerated_label" = "autogenerated_label",
 *     },
 *     "auto_label_source" = "name",
 *     "method_parameter" = {
 *       "name" = "name",
 *       "description" = "description",
 *       "required" = "required",
 *     },
 *     "item" = {
 *       "type" = "param_type",
 *       "format" = "format",
 *       "items" = "api_param_item",
 *       "collection_format" = "collection_format",
 *       "default" = "param_default",
 *     },
 *     "allow_empty_value" = "allow_empty_value",
 *     "api_ref" = "api_ref",
 *     "api_version_tag" = "api_version_tag",
 *   },
 *   bundle_entity_type = "api_query_param_type",
 *   bundle_label = @Translation("API HTTP Method Query Parameter type"),
 *   revision_table = "api_query_param_revision",
 *   revision_data_table = "api_query_param_field_revision",
 *   show_revision_ui = TRUE,
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_user",
 *     "revision_created" = "revision_created",
 *     "revision_log_message" = "revision_log",
 *   },
 *   links = {
 *     "canonical" = "/api_query_param/{api_query_param}",
 *     "add-page" = "/api_query_param/add",
 *     "add-form" = "/api_query_param/add/{api_query_param_type}",
 *     "edit-form" = "/api_query_param/{api_query_param}/edit",
 *     "delete-form" = "/api_query_param/{api_query_param}/delete",
 *     "collection" = "/admin/content/api_query_param",
 *     "version-history" = "/api_query_param/{api_query_param}/revisions",
 *     "revision" = "/api_query_param/{api_query_param}/revisions/{api_query_param_revision}/view",
 *     "revision_revert" = "/api_query_param/{api_query_param}/revisions/{api_query_param_revision}/revert",
 *     "revision_delete" = "/api_query_param/{api_query_param}/revisions/{api_query_param_revision}/delete",
 *     "multiple_delete_confirm" = "/admin/content/api_query_param/delete",
 *     "translation_revert" = "/api_query_param/{api_query_param}/revisions/{api_query_param_revision}/revert/{langcode}",
 *   },
 *   translatable = TRUE,
 * )
 */
class APIQueryParam extends RevisionableContentEntityBase implements APIQueryParamInterface {

  use EntityChangedTrait;
  use AutoLabelTrait;
  use MethodParameterTrait;
  use ItemTrait;
  use AllowEmptyValueTrait;
  use APIRefTrait;
  use APIVersionTagRefTrait;
  use URLRouteParametersTrait;

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    // Generate auto label.
    $this->autoLabelPreSave();
  }

  /**
   * {@inheritdoc}
   */
  public function preSaveRevision(EntityStorageInterface $storage, \stdClass $record) {
    parent::preSaveRevision($storage, $record);

    if (!$this->isNewRevision() && isset($this->original) && (!isset($record->revision_log) || $record->revision_log === '')) {
      // If we are updating an existing APIQueryParam without adding a new
      // revision, we need to make sure $entity->revision_log is reset whenever
      // it is empty. Therefore, this code allows us to avoid clobbering an
      // existing log entry with an empty one.
      $record->revision_log = $this->original->revision_log->value;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    /** @var \Drupal\Core\Field\BaseFieldDefinition[] $fields */
    $fields = parent::baseFieldDefinitions($entity_type);

    // Add the auto label field.
    $fields += static::autoLabelBaseFieldDefinitions($entity_type);

    // Add the method parameter fields.
    $fields += static::methodParameterBaseFieldDefinitions($entity_type);

    // Add the allow empty value field.
    $fields += static::allowEmptyValueBaseFieldDefinitions($entity_type);

    // Add the item fields.
    $fields += static::itemBaseFieldDefinitions($entity_type);
    $fields['api_param_item']->setDescription(t('API Parameter Item referenced from API Query Parameter.'));

    // Add the API Reference field.
    $fields += static::apiRefBaseFieldDefinitions($entity_type);
    $fields['api_ref']->setDescription(t('API Reference referenced from API Query Parameter.'));

    // Add the API Version Tag field.
    $fields += static::apiVersionTagBaseFieldDefinitions($entity_type);
    $fields['api_version_tag']->setDescription(t('API Version Tag referenced from API Query Parameter.'));

    $fields['id']->setDescription(t('The API HTTP Method Query Parameter ID.'));

    $fields['uuid']->setDescription(t('The API HTTP Method Query Parameter UUID.'));

    $fields['vid']->setDescription(t('The API HTTP Method Query Parameter revision ID.'));

    $fields['langcode']->setDescription(t('The API HTTP Method Query Parameter language code.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the API HTTP Method Query Parameter was last edited.'))
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    $label = parent::label();
    if (empty($label)) {
      $label = $this->generateAutoLabel();
    }
    return $label;
  }

}
